# This file is part of ROBOTGAME
#
# ROBOTGAME is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# ROBOTGAME is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# ROBOTGAME.  If not, see <http://www.gnu.org/licenses/>.
#
# ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' '
#
#                          player.py
#                     --------------------
#       date created : Tue Aug 7 2012
#          copyright : (C) 2012 Sakse Dalum
#      maintained by : Sakse Dalum <don_s@hongabar.org>

"""
The player.
"""

import pygame
import re
import os

import worldobject
import boulder
import lever

class Player(worldobject.WorldObject):
    def __init__(self, level, x, y, z=1, movable=True):
        self.__dict__.update(locals())
        worldobject.WorldObject.__init__(self, level, x, y,
                                         z=z, movable=movable)

        self.anim_root = 'idle'
        self.anim = 'right'
        self.frame = 0
        self.anim_speed = 15

        self.working = False

        self.ignore_list.append(boulder.Boulder)

        self.imgs = dict([(k[6:], v) for (k, v) in self.level.imgs.iteritems()
                          if k.startswith('robot_')])

    def direction_as_string(self):
        if self.direction == (1, 0):
            return "right"
        if self.direction == (-1, 0):
            return "left"
        if self.direction == (0, -1):
            return "up"
        if self.direction == (0, 1):
            return "down"

    def touch(self, touch_x, touch_y):
        for obj in self.level.objects:
            if (obj.x == self.x + touch_x * self.tile_x
                and obj.y == self.y + touch_y * self.tile_y
                and obj != self):

                # Concerning levers
                if type(obj) == lever.Lever:
                    if not obj.at_rest:
                        return

                    obj.use(self)
                    self.working = True

                    # List all possible combinations ...
                    lever_direction = (
                        'up' if obj.anim == 'lever_updown' and not obj.setting
                        else
                        'right' if obj.anim == 'lever_leftright'
                                and not obj.setting
                        else
                        'down' if obj.anim == 'lever_updown' and obj.setting
                        else
                        'left')

                    self.anim_root = 'lever'
                    self.frame = 0
                    self.anim = (
                        self.direction_as_string() + '_' + lever_direction)
                else:
                    obj.use(self)

    def update(self, e, t, dt):
        if not self.working:
            self.anim_root = 'carry' if self.holding else 'idle'

            keys = pygame.key.get_pressed()
            if keys[pygame.K_UP]:
                if not self.holding and not self.is_moving:
                    self.direction = (0, -1)
                    self.anim = self.direction_as_string()
                self.move(0, -1)
            elif keys[pygame.K_DOWN]:
                if not self.holding and not self.is_moving:
                    self.direction = (0, 1)
                    self.anim = self.direction_as_string()
                self.move(0, 1)
            elif keys[pygame.K_RIGHT]:
                if not self.holding and not self.is_moving:
                    self.direction = (1, 0)
                    self.anim = self.direction_as_string()
                self.move(1, 0)
            elif keys[pygame.K_LEFT]:
                if not self.holding and not self.is_moving:
                    self.direction = (-1, 0)
                    self.anim = self.direction_as_string()
                self.move(-1, 0)

            for event in e:
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_SPACE:
                        self.touch(*self.direction)

        else:
            self.working = not (self.frame == len(
                    self.imgs[self.anim_root + '_' + self.anim]) - 1)
            if not self.working:
                self.anim = self.direction_as_string()
                self.anim_root = 'carry' if self.holding else 'idle'

        # Update the animation
        if not self.working:
            self.frame = ((self.frame + self.anim_speed * dt) %
                          len(self.imgs[self.anim_root + '_' + self.anim]))
        else:
            self.frame = min(
                self.frame + self.anim_speed * dt,
                len(self.imgs[self.anim_root + '_' + self.anim]) - 1)


        worldobject.WorldObject.update(self, e, t, dt)

    def draw(self, window):
        self.img = self.imgs[self.anim_root + '_' + self.anim][int(self.frame)]
        self.img.set_alpha(128)
        offset_x = (96 if self.anim_root == 'lever' and self.anim in
                    ['left_up', 'left_left', 'left_down', 'left_right',
                     'down_right', 'up_left']
                    else 32)
        window.blit(self.img, (self.x - offset_x - self.level.camera_x,
                               self.y - self.img.get_size()[1] + 24
                               - self.level.camera_y))

